#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: LGPL-3.0-or-later
#
"""giradischi module."""

from pathlib import Path

__version__ = "1.0.0"

module_path = Path(__file__).parent
current_path = Path.cwd()
